// --------------------------------------------------------------------------------------
//
// HanoiIR2.CS                                                     Ulrich Mller 31.10.05
//                                                                 PCS1700   C# Final SP2
// Hanoi mit dem Industry II SulenRobot und 20 mm gelben Tonnen
// bertragung von HanoiRobot40 MagnetRoboter
//
// Endtaster werden linksdrehend (Klick auf L beim Interface Panel) angefahren
// Alle Positionen ab Endtaster in Impulsen
// Die Scheiben (Tonnen) stehen auf gelben Kstchen
//
// --------------------------------------------------------------------------------------

using System;
using cs = System.Console;
using FishFace40;

namespace CSTestConsole
{
	class Rahmen
	{
		static FishFace ft = new FishFace();
		const int mSaule   = 1;
		const int mArmH      = 2;
		const int mArm     = 3;
		const int mGreifer = 4;
		const int eSauleEnde = 1;
		const int eSauleImpuls = 2;
		const int eArmOben     = 5;
		const int eArmImpuls   = 6;
		const int eGreiferAuf  = 7;
		const int eGreiferImpuls = 8;
		const int PileA          = 0;
		const int PileB          = 2;
		const int PileC          = 1;
		const int ArmFrei        = 20;
		const int ScheibenH      = 35;
		const int GreiferZu      = 26;
		
		int SaulenPos;
		int ArmPos;
		int[,] ScheibenK = {{24,64,96},{31,135,135}};

		[STAThread]
		static void Main(string[] args)	{
			Rahmen rt = new Rahmen();
			cs.WriteLine("--- HanoiIR2-Robot gestartet ---");
			try {
				ft.OpenInterface(IFTypen.ftROBO_first_USB, 0);
				rt.Action();
			}
			catch(FishFaceException eft) {
				cs.WriteLine(eft.Message);
			}
			finally {
				ft.CloseInterface();
				cs.WriteLine("--- HanoiIR2-Robot beendet ---");
				cs.Read();
			}
		}
		private void Action() {
			cs.WriteLine("Robot fhrt auf Grundstellung");
			Grundstellung();
			Hanoi(3, PileA, PileB, PileC);
			ft.Pause(1234);
			SauleNach(112);
		}
		
		// --- Die Kern-Methode mit der Hanoi-Lsung -------------
		private void Hanoi(int n, int PileA, int PileB, int PileC) {
			if(n == 1) {
				cs.WriteLine("Scheibe von " + PileA + " -> " + PileC);
				Ziehe(PileA, PileC);
			}
			else {
				Hanoi(n-1, PileA, PileC, PileB);
				cs.WriteLine("Scheibe von " + PileA + " -> " + PileC);
				Ziehe(PileA, PileC);
				Hanoi(n-1, PileB, PileA, PileC);
			}
		}
		
		// --- Ziehen einer Scheibe VON -> NACH -------
		private void Ziehe(int VonPile, int NachPile) {
			Hole(VonPile);
			Bringe(NachPile);
		}
		
		// --- Holen einer Scheibe vom Quellstapel ----
		//     Der Greifer ist in Hhe ArmFrei
		private void Hole(int Pile) {
			SauleNach(ScheibenK[0, Pile]);
			ArmNach(ScheibenK[1, Pile]+ScheibenH);
			ScheibeGreifen();
			ScheibenK[1, Pile] = ArmPos;
			ArmNach(ArmFrei);
		}

		// --- Bringen einer Scheibe zum Zielstapel ---
		//     Greifer ist in Hhe ArmFrei
		private void Bringe(int Pile) {
			SauleNach(ScheibenK[0, Pile]);
			ArmNach(ScheibenK[1, Pile]);
			ScheibeLegen();
			ScheibenK[1, Pile] = ArmPos - ScheibenH;
			ArmNach(ArmFrei);
		}
		
		// --- Fahren auf Grundstellung --------------
		private void Grundstellung() {
			ScheibeLegen();               // --- Greifer auf
			ArmPos = 999;                 // --- Arm auf Hhe ArmFrei
			ArmNach(0);
			ArmPos = 0;
			ArmNach(ArmFrei);
			ft.SetMotor(mArmH, (int)Dir.Left); // --- Horz. Arm auf Pos. 0
			ft.WaitForInput(Inp.I3, true);
			ft.SetMotor(mArmH, (int)Dir.Off);
			SaulenPos = 999;              // --- Sule auf Position PileA
			SauleNach(0);
			SauleNach(ArmFrei);
			SaulenPos = 0;
		}
		
		// --- Fahren der Sule auf ZielPosition -----------------
		private void SauleNach(int ZielPos) {
			if(SaulenPos < ZielPos) {
				ft.SetMotor(mSaule, (int)Dir.Right, (int)Speed.Full, ZielPos-SaulenPos);
				ft.WaitForMotors(0, mSaule);
			}
			else {
				ft.SetMotor(mSaule, (int)Dir.Left, (int)Speed.Full, SaulenPos-ZielPos);
				ft.WaitForMotors(0, mSaule);
			}
			SaulenPos = ZielPos + ft.GetCounter(eSauleImpuls);
		}
		
		// --- Vertikales Fahren des Arms auf Zielhhe -------------
		private void ArmNach(int ZielPos) {
			if(ArmPos < ZielPos) {
				ft.SetMotor(mArm, (int)Dir.Right, (int)Speed.Full, ZielPos-ArmPos);
				ft.WaitForMotors(0, mArm);
			}
			else {
				ft.SetMotor(mArm, (int)Dir.Left, (int)Speed.Full, ArmPos-ZielPos);
				ft.WaitForMotors(0, mArm);
			}
			ArmPos = ZielPos + ft.GetCounter(eArmImpuls);
		}
		
		// --- Ablegen der Scheibe auf aktuelles Pile ----
		private void ScheibeLegen() {
			ft.SetMotor(mGreifer, (int)Dir.Left, (int)Speed.Full);
			ft.WaitForInput(eGreiferAuf);
			ft.SetMotor(mGreifer, (int)Dir.Aus);
		}
		
		// --- Aufnehmen der Scheibe ---------------------
		private void ScheibeGreifen() {
			ft.SetMotor(mGreifer, (int)Dir.Right, (int)Speed.Full, GreiferZu);
			ft.WaitForMotors(0, mGreifer);
		}
	}
}
