# ---------------------------------------------------------------------------------
#
# HanoiRobot.PY                                             Ulrich Mueller  5.10.03
#                                                           PCS1700    Python 2.2.3
# Loesung des "Turm von Hanoi Problems mit einem Roboter
#
# ---------------------------------------------------------------------------------

from FishFa30 import *
   
# -------------- Klasse HanoiRobot ---------------------------------------------

class HanoiRobot(FishFace):

# --- Eigenschaften ------
  mSaule       = 1
  mArm         = 2
  mGreifer     = 3
  eArmOben     = 3
  eArmUnten    = 4
  eSauleImpuls = (mSaule-1)*2+2
  eSauleEnde   = (mSaule-1)*2+1
  PosA         = 30
  PosB         = 75
  PosC         = 120

  def GrundStellung(self):
    "Fahren auf Grundstellung"
    self.SetMotor(self.mGreifer, self.Aus)
    self.ArmHeben()
    self.RobotPos = 999
    self.SauleNach(0)
    self.SauleNach(self.PosA)

  def SauleNach(self, ZielPos):
    "Fahren auf Zielposition"
    if(self.RobotPos < ZielPos):
      self.SetMotor(self.mSaule, self.Rechts, 15, ZielPos-self.RobotPos)
      self.WaitForMotors(0, (self.mSaule,))
    else:
      self.SetMotor(self.mSaule, self.Links, 15, self.RobotPos-ZielPos)
      self.WaitForMotors(0, (self.mSaule,))
    self.RobotPos = ZielPos + self.GetCounter(self.eSauleImpuls)

  def ArmSenken(self):
    "Senken des Greiferarmes"
    self.SetMotor(self.mArm, self.Links)
    self.WaitForInput(self.eArmUnten)
    self.SetMotor(self.mArm, self.Aus)

  def ArmHeben(self):
    "Heben des Greiferarms"
    self.SetMotor(self.mArm, self.Rechts)
    self.WaitForInput(self.eArmOben)
    self.SetMotor(self.mArm, self.Aus)

  def ScheibeLegen(self):
    "Ablegen der Scheibe"
    self.SetMotor(self.mGreifer, self.Aus)

  def ScheibeGreifen(self):
    "Aufnehmen der Scheibe"
    self.SetMotor(self.mGreifer, self.Ein)

# ---- Hanoi-Logik ---------------------------------------

def Hanoi(n, TAnf, TMit, TEnd):
  "Das Kern-Programm mit der Hanoi-Lsung "
  if(n == 1):
    print "Scheibe von %d -> %d" % (TAnf, TEnd)
    Ziehe(TAnf, TEnd)
  else:
    Hanoi(n-1, TAnf, TEnd, TMit)
    print "Scheibe von %d -> %d" % (TAnf, TEnd)
    Ziehe(TAnf, TEnd)
    Hanoi(n-1, TMit, TAnf, TEnd)

def Ziehe(Von, Nach):
  "Ziehen einer Scheibe VON -> NACH"
  Hole(Von)
  Bringe(Nach)

def Hole(Pos):
  "Holen einer Scheibe vom Quellstapel"
  hr.SauleNach(Pos)
  hr.ArmSenken()
  hr.ScheibeGreifen()
  hr.ArmHeben()

def Bringe(Pos):
  "Bringen einer Scheibe zum Zielstapel"
  hr.SauleNach(Pos)
  hr.ArmSenken()
  hr.ScheibeLegen()
  hr.ArmHeben()

# ----
# ---- Hauptprogramm --------------
# ----

hr = HanoiRobot()

try:
  hr.OpenInterface("COM2")
  print "---- Faehrt auf Grundstellung ----"
  hr.GrundStellung()
  Hanoi(3, hr.PosA, hr.PosB, hr.PosC)
  print "---- FINIS ----"

except FishFaceException:
  print "FishFace Fehler"

hr.CloseInterface()
